import sys
sys.path.append('..')
from scrython.foundation import FoundationObject

class RulingsObject(FoundationObject):
    """
    Master class for all rulings objects.

    Args:
        format (string, optional): Returns data in the specified method. Defaults to JSON.
        face (string, optional): 
            If you're using the `image` format, this will specify if you want the front or back face.
        version (string, optional):
            If you're using the `image` format, this will specify if you want the small, normal, large, etc version of the image.
        pretty (string, optional):
            Returns a prettier version of the json object. Note that this may break functionality with Scrython.

    """

    def object(self):
        """Returns the type of object it is
        (card, error, etc)
        
        Returns:
            string
        """
        super(RulingsObject, self)._checkForKey('object')

        return self.scryfallJson['object']

    def has_more(self):
        """True if there is more than one page of results
        
        Returns:
            boolean: True if there are more results
        """
        super(RulingsObject, self)._checkForKey('has_more')

        return self.scryfallJson['has_more']

    def data(self, index=None, key=None):
        """The data returned from the query

        Acceptable keys:
            object (string): The type of object for a given ruling.
            source (string): The source of the ruling.
            published_at (string): The date when the ruling was published.
            comment (string): The effective ruling.

        Args:
            index (integer, optional): Defaults to None. Access a specific index.
            key (string, optional): Defaults to None. Returns the value of the given key. Requires the `index` argument.
        
        Returns:
            List: The full list of data.
            Dictionary: If given an index
            String: If given an index and key.
        """
        super(RulingsObject, self)._checkForKey('data')

        if index is not None:
            if key is not None:
                super(RulingsObject, self)._checkForTupleKey('data', index, key)
                return self.scryfallJson['data'][index][key]

            return self.scryfallJson['data'][index]

        return self.scryfallJson['data']

    def data_length(self):
        """The length of the `data` list.
        
        Returns:
            Integer
        """
        super(RulingsObject, self)._checkForKey('data')

        return len(self.scryfallJson['data'])



from .catalogs_object import CatalogsObject


class AbilityWords(CatalogsObject):
    """
    catalogs/ability-words

    Catalog object for all known ability words

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> catalog = scrython.catalog.KeywordAbilities()
        >>> catalog.data()
    """

    def __init__(self):
        self._url = 'catalog/ability-words?'
        super(AbilityWords, self).__init__(self._url)



import sys
sys.path.append('..')
from scrython.foundation import FoundationObject

class Sets(FoundationObject):
    """
    /sets

    Args:
        code (string): The 3 letter code of the set
        format (string, optional):
            Returns data in the specified method. Defaults to JSON.
        pretty (string, optional):
            Returns a prettier version of the json object. Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> set = scrython.sets.Sets()
        >>> set.data(3, "name")
    """
    def __init__(self):
        self._url = 'sets?'
        super(Sets, self).__init__(self._url)

    def object(self):
        """Returns the type of object it is
        (card, error, etc)
        
        Returns:
            string
        """
        super(Sets, self)._checkForKey('object')

        return self.scryfallJson['object']

    def has_more(self):
        """True if there are more pages available
        
        Returns:
            boolean
        """
        super(Sets, self)._checkForKey('has_more')

        return self.scryfallJson['has_more']

    def data(self, index=None, key=None):
        """The data returned from the query

        Acceptable keys:
            object (string): The set object.
            code (string): The three letter set code of the set.
            mtgo_code (string): The mtgo equivalent of `code()`.
            name (string): The full name of the set.
            set_type (string): The type of the set (expansion, commander, etc)
            released_at (string): The date the set was launched.
            block_code (string): The the letter code for the block the set was in.
            block (string): The full name of the block a set was in.
            parent_set_code (string): The set code for the parent set.
            card_count (integer): The number of cards in the set.
            digital (boolean): True if this set is only featured on MTGO.
            foil_only (boolean): True if this set only has foils.
            icon_svg_uri (string): A URI to the SVG of the set symbol.
            search_uri (string): The scryfall API url for the search.

        Args:
            index (integer, optional): Defaults to None. Access a specific index.
            key (string, optional): Defaults to None. Returns the value of the given key. Requires the `index` argument.
        
        Returns:
            List: The full list of data.
            Dictionary: If given an index
            String: If given an index and key.
        """
        super(Sets, self)._checkForKey('data')

        if index is not None:
            if key is not None:
                super(Sets, self)._checkForTupleKey('data', index, key)
                return self.scryfallJson['data'][index][key]

            return self.scryfallJson['data'][index]

        return self.scryfallJson['data']

    def data_length(self):
        """The length of the data returned
        
        Returns:
            integer
        """
        super(Sets, self)._checkForKey('data')

        return len(self.scryfallJson['data'])


from .cards_object import CardsObject

class Collector(CardsObject):
    """
    cards/collector
    Get a card by collector number.

    Args:
        code (string): This is the 3 letter code for the set
        collector_number (string): This is the collector number of the card
        lang (string, optional): Defaults to 'en'. A 2-3 character language code.

    Returns:
        N/A

    Raises:
        Exception: If the 'code' parameter is not provided.
        Exception: If the 'collector_number' parameter is not provided.
        Exception: If the object returned is an error.

    Examples:
        >>> card = scrython.cards.Collector(code="exo", collector_number="96")
        >>> card.id()
    """
    def __init__(self, **kwargs):
        if kwargs.get('code') is None:
            raise Exception('No code provided to search by')
        elif kwargs.get('collector_number') is None:
            raise Exception('No collector number provided to search by')

        self.url = 'cards/{}/{}/{}?'.format(
            kwargs.get('code'),
            str(kwargs.get('collector_number')),
            kwargs.get('lang', 'en')
            )
        super(Collector, self).__init__(self.url)



from .catalogs_object import CatalogsObject


class KeywordActions(CatalogsObject):
    """
    catalog/keyword-actions

    Catalog object for all known keyword actions

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> catalog = scrython.catalog.KeywordActions()
        >>> catalog.data()
    """

    def __init__(self):
        self._url = 'catalog/keyword-actions?'
        super(KeywordActions, self).__init__(self._url)



import sys
sys.path.append('..')
from scrython.foundation import FoundationObject
import asyncio
import aiohttp
import urllib.parse
from threading import Thread
import warnings

class BulkData(FoundationObject):
    """
    /bulk-data
    Queries and creates an object relating to the /bulk-data endpoint.

    Args:
        N/A

    Returns:
        object: The Scryfall endpoint object.

    Raises:
        Exception: Raised if Scryfall sends an error object.

    Examples:
        >>> data = scrython.bulk_data.BulkData()
        >>> data.bulk_compressed_size()
    """
    def __init__(self, **kwargs):

        self.url = 'https://api.scryfall.com/bulk-data'
        super(BulkData, self).__init__(self.url, True)

    def object(self):
        """Returns the type of object it is.
        (card, error, etc)
        
        Returns:
            string: The type of object
        """
        super(BulkData, self)._checkForKey('object')

        return self.scryfallJson['object']

    def has_more(self):
        """True if there is more than one page of results
        
        Returns:
            boolean: True if there are more results
        """
        super(BulkData, self)._checkForKey('has_more')

        return self.scryfallJson['has_more']

    def data(self):
        """A list of all types of types returned by the endpoints
        
        Returns:
            list: List of all types
        """
        super(BulkData, self)._checkForKey('data')

        return self.scryfallJson['data']

    def bulk_object(self, num):
        """Returns the type of object the specified index is
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: The type of object
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'object')

        return self.scryfallJson['data'][num]['object']

    def bulk_id(self, num):
        """The unique ID of the bulk item
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: The Scryfall id of the object
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'id')

        return self.scryfallJson['data'][num]['id']

    def bulk_type(self, num):
        """The type of bulk data
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: The type of the data item
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'type')

        return self.scryfallJson['data'][num]['type']

    def bulk_updated_at(self, num):
        """The time the item was last updated
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: Timestamp
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'updated_at')

        return self.scryfallJson['data'][num]['updated_at']

    def bulk_name(self, num):
        """The name of the type of bulk data object
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: The name of the data item
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'name')

        return self.scryfallJson['data'][num]['name']

    def bulk_description(self, num):
        """A description of the object
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: The description of the data item
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'description')

        return self.scryfallJson['data'][num]['description']

    def bulk_compressed_size(self, num, human_readable=False):
        """The size of the file in bytes
        
        Args:
            num (int): The index of the object in the `data` key
            human_readable (bool, optional): Defaults to False. Converts the bytes into a human readable format
        
        Returns:
            integer: Returns integer by default. 
            string: If human_readable is True, returns a string.
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'compressed_size')

        if human_readable:
            before = self.scryfallJson['data'][num]['compressed_size']

            for unit in ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB']:
                if abs(before) < 1024.0:
                    return '{:3.1f}{}'.format(before, unit)
                before /= 1024.0

            return '{:.1f}{}'.format(before, 'YiB')

        return self.scryfallJson['data'][num]['compressed_size']

    def bulk_permalink_uri(self, num):
        warnings.warn("This method has been renamed to bulk_uri as per https://scryfall.com/blog/updates-to-bulk-data-and-cards-deprecation-notice-217", DeprecationWarning)
        return self.bulk_uri(num)

    def bulk_uri(self, num):
        """The URL that hosts the bulk file
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: A URI to download the compressed data
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'uri')

        return self.scryfallJson['data'][num]['uri']

    def bulk_content_type(self, num):
        """The MIME type of the file
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: The MIME type
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'content_type')

        return self.scryfallJson['data'][num]['content_type']

    def bulk_content_encoding(self, num):
        """The encoding of the file
        
        Args:
            num (int): The index of the object in the `data` key
        
        Returns:
            string: The encoding of the file
        """
        super(BulkData, self)._checkForTupleKey('data', num, 'content_encoding')

        return self.scryfallJson['data'][num]['content_encoding']



from .artifact_types import ArtifactTypes
from .card_names import CardNames
from .creature_types import CreatureTypes
from .enchantment_types import EnchantmentTypes
from .land_types import LandTypes
from .loyalties import Loyalties
from .planeswalker_types import PlaneswalkerTypes
from .powers import Powers
from .spell_types import SpellTypes
from .toughnesses import Toughnesses
from .watermarks import Watermarks
from .word_bank import WordBank
from .artist_names import ArtistNames
from .keyword_abilities import KeywordAbilities
from .keyword_actions import KeywordActions
from .ability_words import AbilityWords

__all__ = [
    'ArtifactTypes',
    'CardNames',
    'CreatureTypes',
    'EnchantmentTypes',
    'LandTypes',
    'Loyalties',
    'PlaneswalkerTypes',
    'Powers',
    'SpellTypes',
    'Toughnesses',
    'Watermarks',
    'WordBank',
    'ArtistNames',
    'KeywordAbilities',
    'KeywordActions',
    'AbilityWords'
]



from .catalogs_object import CatalogsObject

class LandTypes(CatalogsObject):
    """
    catalogs/land-types

    Catalog object for all known land types.

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> catalog = scrython.catalog.LandTypes()
        >>> catalog.data()
    """
    def __init__(self):
        self._url = 'catalog/land-types?'
        super(LandTypes, self).__init__(self._url)



import sys
sys.path.append('..')
from scrython.foundation import FoundationObject

class CatalogsObject(FoundationObject):
    """
    Master object for all catalog objects.

    Args:
        format (string, optional):
            Defaults to 'json'.
            Returns data in the specified method.
        pretty (string, optional):
            Defaults to empty string.
            Returns a prettier version of the json object.
            Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        N/A
    """

    def object(self):
        """Returns the type of object it is
        (card, error, etc)
        
        Returns:
            string
        """
        super(CatalogsObject, self)._checkForKey('object')

        return self.scryfallJson['object']

    def uri(self):
        """The API URI for the endpoint you've called.
        
        Returns:
            string
        """
        super(CatalogsObject, self)._checkForKey('uri')

        return self.scryfallJson['uri']

    def total_values(self):
        """The number of items in `data()`
        
        Returns:
            integer
        """
        super(CatalogsObject, self)._checkForKey('total_values')

        return self.scryfallJson['total_values']

    def data(self):
        """A list of all types returned by the endpoint
        
        Returns:
            list
        """
        super(CatalogsObject, self)._checkForKey('data')

        return self.scryfallJson['data']



import sys
sys.path.append('..')
from scrython.foundation import FoundationObject
import aiohttp
import asyncio
import urllib.parse
from threading import Thread

class CardsObject(FoundationObject):
    """
    Master class that all card objects inherit from.

    Args:
        format (string, optional):
            Defaults to 'json'.
            Returns data in the specified method.
        face (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify if you want the front or back face.
        version (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify if you want the small, normal,
            large, etc version of the image.
        pretty (string, optional):
            Defaults to empty string.
            Returns a prettier version of the json object.
            Note that this may break functionality with Scrython.

    Raises:
        Exception: If the object returned is an error.
    """

    def object(self):
        """Returns the type of object it is
        (card, error, etc)
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('object')

        return self.scryfallJson['object']

    def id(self):
        """A unique ID for the returned card object
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('id')

        return self.scryfallJson['id']

    def multiverse_ids(self):
        """The official Gatherer multiverse ids of the card
        
        Returns:
            list
        """
        super(CardsObject, self)._checkForKey('multiverse_ids')

        return self.scryfallJson['multiverse_ids']

    def mtgo_id(self):
        """The official MTGO id of the of the card
        
        Returns:
            integer: The Magic Online id of the card
        """
        super(CardsObject, self)._checkForKey('mtgo_id')

        return self.scryfallJson['mtgo_id']

    def mtgo_foil_id(self):
        """The corresponding MTGO foil ID of the card
        
        Returns:
            integer: The Magic Online foil id of the card
        """
        super(CardsObject, self)._checkForKey('mtgo_foil_id')

        return self.scryfallJson['mtgo_foil_id']

    def tcgplayer_id(self):
        """The `productId` of the card on TCGplayer.

        Returns:
            integer: The TCGplayer id of the card
        """
        super(CardsObject, self)._checkForKey('tcgplayer_id')

        return self.scryfallJson['tcgplayer_id']

    def tcgplayer_etched_id(self):
        """The `etched_id` of the card on TCGplayer.

        Returns:
            integer: The TCGplayer etched id of the card
        """
        super(CardsObject, self)._checkForKey('tcgplayer_etched_id')

        return self.scryfallJson['tcgplayer_etched_id']

    def name(self):
        """The oracle name of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('name')

        return self.scryfallJson['name']

    def uri(self):
        """The Scryfall API uri for the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('uri')

        return self.scryfallJson['uri']

    def scryfall_uri(self):
        """The full Scryfall page of the card
        As if it was a URL from the site.
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('scryfall_uri')

        return self.scryfallJson['scryfall_uri']

    def layout(self):
        """The image layout of the card. (normal, transform, etc)
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('layout')

        return self.scryfallJson['layout']

    def highres_image(self):
        """Determine if a card has a highres scan available
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('highres_image')

        return self.scryfallJson['highres_image']

    def image_uris(self, index=0, image_type=None):
        """All image uris of the card in various qualities

        An index and an image type must be supplied a single uri.

        If the card has additional faces, the returned dict will
        default to the front of the card.

        Returns:
            dict: If given no arguments
            string: If given an index and image_type

        Raises:
            Exception: If given no index
            KeyError: If the given image type is not a known type
        """

        layouts = {
            'normal': lambda num: self.scryfallJson['image_uris'],
            'split': lambda num: self.scryfallJson['image_uris'],
            'flip': lambda num: self.scryfallJson['image_uris'],
            'transform': lambda num: self.scryfallJson['card_faces'][num]['image_uris'],
            'meld': lambda num: self.scryfallJson['image_uris'],
            'leveler': lambda num: self.scryfallJson['image_uris'],
            'saga': lambda num: self.scryfallJson['image_uris'],
            'class': lambda num: self.scryfallJson['image_uris'],
            'planar': lambda num: self.scryfallJson['image_uris'],
            'scheme': lambda num: self.scryfallJson['image_uris'],
            'vanguard': lambda num: self.scryfallJson['image_uris'],
            'token': lambda num: self.scryfallJson['image_uris'],
            'double_faced_token': lambda num: self.scryfallJson['card_faces'][num]['image_uris'],
            'emblem': lambda num: self.scryfallJson['image_uris'],
            'augment': lambda num: self.scryfallJson['image_uris'],
            'host': lambda num: self.scryfallJson['image_uris'],
            'adventure': lambda num: self.scryfallJson['image_uris'],
            'modal_dfc': lambda num: self.scryfallJson['card_faces'][num]['image_uris']
        }

        image_types = {
            'small': lambda d: d['small'],
            'normal': lambda d: d['normal'],
            'large': lambda d: d['large'],
            'png': lambda d: d['png'],
            'art_crop': lambda d: d['art_crop'],
            'border_crop': lambda d: d['border_crop']
        }

        images_dict = layouts.get(self.scryfallJson['layout'])

        uri = image_types.get(image_type)

        if index == 0 and image_type is None:
            return images_dict(0)
        elif not isinstance(index, int):
            raise Exception('You must supply an index to get a uri')
        elif image_type not in list(image_types.keys()):
            raise KeyError('Image type not in known types')

        return uri(images_dict(index))

    def cmc(self):
        """A float of the converted mana cost of the card
        
        Returns:
            float: The cmc of the card
        """
        super(CardsObject, self)._checkForKey('cmc')

        return self.scryfallJson['cmc']

    def type_line(self):
        """The full type line of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('type_line')

        return self.scryfallJson['type_line']

    def oracle_text(self):
        """The official oracle text of a card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('oracle_text')

        return self.scryfallJson['oracle_text']

    def mana_cost(self):
        """The full mana cost using shorthanded mana symbols
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('mana_cost')

        return self.scryfallJson['mana_cost']

    def colors(self):
        """A list of strings with all colors found in the mana cost
        
        Returns:
            list
        """
        super(CardsObject, self)._checkForKey('colors')

        return self.scryfallJson['colors']

    def color_identity(self):
        """A list of strings with all colors found on the card itself
        
        Returns:
            list
        """
        super(CardsObject, self)._checkForKey('color_identity')

        return self.scryfallJson['color_identity']

    def legalities(self):
        """A dictionary of all formats and their legality
        
        Returns:
            dict
        """
        super(CardsObject, self)._checkForKey('legalities')

        return self.scryfallJson['legalities']

    def reserved(self):
        """Returns True if the card is on the reserved list
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('reserved')

        return self.scryfallJson['reserved']

    def reprint(self):
        """Returns True if the card has been reprinted before
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('reprint')

        return self.scryfallJson['reprint']

    def set_code(self):
        """The 3 letter code for the set of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('set')

        return self.scryfallJson['set']

    def set_name(self):
        """The full name for the set of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('set_name')

        return self.scryfallJson['set_name']

    def set_uri(self):
        """The API uri for the full set list of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('set_uri')

        return self.scryfallJson['set_uri']

    def set_search_uri(self):
        """Same output as set_uri
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('set_search_uri')

        return self.scryfallJson['set_search_uri']

    def scryfall_set_uri(self):
        """The full link to the set on Scryfall
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('scryfall_set_uri')

        return self.scryfallJson['scryfall_set_uri']

    def rulings_uri(self):
        """The API uri for the rulings of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('rulings_uri')

        return self.scryfallJson['rulings_uri']

    def prints_search_uri(self):
        """A link to where you can begin paginating all re/prints for this card on Scryfall’s API
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('prints_search_uri')

        return self.scryfallJson['prints_search_uri']

    def collector_number(self):
        """The collector number of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('collector_number')

        return self.scryfallJson['collector_number']

    def digital(self):
        """Returns True if the card is the digital version
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('digital')

        return self.scryfallJson['digital']

    def rarity(self):
        """The rarity of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('rarity')

        return self.scryfallJson['rarity']

    def illustration_id(self):
        """The related id of the card art
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('illustration_id')

        return self.scryfallJson['illustration_id']

    def artist(self):
        """The artist of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('artist')

        return self.scryfallJson['artist']

    def frame(self):
        """The year of the card frame
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('frame')

        return self.scryfallJson['frame']

    def frame_effects(self):
        """The card's frame effect, if any. (miracle, nyxtouched, etc.)
        
        Returns:
            list: The card's frame effects.
        """
        super(CardsObject, self)._checkForKey('frame_effects')

        return self.scryfallJson['frame_effects']

    def full_art(self):
        """Returns True if the card is considered full art
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('full_art')

        return self.scryfallJson['full_art']

    def border_color(self):
        """The color of the card border
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('border_color')

        return self.scryfallJson['border_color']

    def edhrec_rank(self):
        """The rank of the card on edhrec.com
        
        Returns:
            int: The rank of the card on edhrec.co
        """
        super(CardsObject, self)._checkForKey('edhrec_rank')

        return self.scryfallJson['edhrec_rank']

    def prices(self, mode):
        """Returns prices from modes `usd`, `usd_foil`, `usd_etched`, `usd_glossy`, `eur`, and `tix`
        
        Args:
            mode (string): The prices to get
        
        Raises:
            KeyError: If the mode parameter does not match a known key
        
        Returns:
            float: The prices as a float
        """
        modes = ['usd', 'usd_foil', 'usd_etched', 'usd_glossy', 'eur', 'tix']
        if mode not in modes:
            raise KeyError("{} is not a key.".format(mode))

        super(CardsObject, self)._checkForKey('prices', mode)

        return self.scryfallJson['prices'][mode]

    def related_uris(self):
        """A dictionary of related websites for this card
        
        Returns:
            dict
        """
        super(CardsObject, self)._checkForKey('related_uris')

        return self.scryfallJson['related_uris']

    def purchase_uris(self):
        """A dictionary of links to purchase the card
        
        Returns:
            dict
        """
        super(CardsObject, self)._checkForKey('purchase_uris')

        return self.scryfallJson['purchase_uris']

    def life_modifier(self):
        """This is the cards life modifier value, assuming it's a Vanguard card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('life_modifier')

        return self.scryfallJson['life_modifier']

    def hand_modifier(self):
        """This cards hand modifier value, assuming it's a Vanguard card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('hand_modifier')

        return self.scryfallJson['hand_modifier']

    def color_indicator(self, num):
        """An list of all colors found in this card's color indicator
        
        Returns:
            list
        """
        self._checkForTupleKey('card_faces', num, 'color_indicator')

        return self.scryfallJson['card_faces'][num]['color_indicator']

    def all_parts(self):
        """This this card is closely related to other cards, this property will be an list with it
        
        Returns:
            list
        """
        super(CardsObject, self)._checkForKey('all_parts')

        return self.scryfallJson['all_parts']

    def card_faces(self):
        """If it exists, all parts found on a card's face will be found as an object from this list
        
        Returns:
            list
        """
        super(CardsObject, self)._checkForKey('card_faces')

        return self.scryfallJson['card_faces']

    def watermark(self):
        """The associated watermark of the card, if any
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('watermark')

        return self.scryfallJson['watermark']

    def story_spotlight(self):
        """True if this card is featured in the story
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('story_spotlight')

        return self.scryfallJson['story_spotlight']

    def power(self):
        """The power of the creature, if applicable
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('power')

        return self.scryfallJson['power']

    def toughness(self):
        """The toughness of the creature, if applicable
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('toughness')

        return self.scryfallJson['toughness']

    def loyalty(self):
        """This card's loyalty. Some loyalties may be X rather than a number
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('loyalty')

        return self.scryfallJson['loyalty']

    def flavor_text(self):
        """The flavor text of the card, if any
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('flavor_text')

        return self.scryfallJson['flavor_text']

    def arena_id(self):
        """The Arena ID of the card, if any
        
        Returns:
            int: The Arena ID of the card, if any
        """
        super(CardsObject, self)._checkForKey('arena_id')

        return self.scryfallJson['arena_id']

    def lang(self):
        """The language of the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('lang')

        return self.scryfallJson['lang']

    def printed_name(self):
        """If the card is in a non-English language, this will be the name as it appears on the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('printed_name')

        return self.scryfallJson['printed_name']

    def printed_type_line(self):
        """If the card is in a non-English language, this will be the type line as it appears on the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('printed_type_line')

        return self.scryfallJson['printed_type_line']

    def printed_text(self):
        """If the card is in a non-English language, this will be the rules text as it appears on the card
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('printed_text')

        return self.scryfallJson['printed_text']

    def oracle_id(self):
        """A unique ID for this card's oracle text
        
        Returns:
            string
        """
        super(CardsObject, self)._checkForKey('oracle_id')

        return self.scryfallJson['oracle_id']

    def foil(self):
        """True if this printing exists in a foil version.

        DEPRECATION NOTICE: This method will be deprecated on Nov 1, 2021.

        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('foil')

        print('WARNING: This method will be deprecated on Nov 1, 2021. Please use the `finishes` method instead.')

        return self.scryfallJson['foil']

    def nonfoil(self):
        """True if this printing does not exist in foil

        DEPRECATION NOTICE: This method will be deprecated on Nov 1, 2021.
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('nonfoil')

        print('WARNING: This method will be deprecated on Nov 1, 2021. Please use the `finishes` method instead.')

        return self.scryfallJson['nonfoil']

    def oversized(self):
        """True if this printing is an oversized card
        
        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('oversized')

        return self.scryfallJson['oversized']

    def games(self):
        """A list of games that this card print is available in.

        Returns:
            array: A list of games
        """
        super(CardsObject, self)._checkForKey('games')

        return self.scryfallJson['games']

    def promo(self):
        """True if this card is a promotional print.

        Returns:
            boolean
        """
        super(CardsObject, self)._checkForKey('promo')

        return self.scryfallJson['promo']

    def released_at(self):
        """The date this card was first released.
        
        Returns:
            string: The date in ISO format
        """
        super(CardsObject, self)._checkForKey('released_at')

        return self.scryfallJson['released_at']

    def preview(self, key=None):
        """Preview information for this card, if any.
        You may pass the name of a valid key to return the value of that key.
        Such as a source_uri.
        
        Args:
            key (string): A key for specific information about the preview.

        Returns:
            dict: If provided no key, the entire dict is returned.
            string: If provided a key, the value of that key is returned.
        """
        super(CardsObject, self)._checkForKey('preview')

        if key in self.scryfallJson['preview']:
            return self.scryfallJson['preview'][key]

        return self.scryfallJson['preview']

    def image_status(self):
        """Provides insight to the status of the images of the card.

        Returns:
            string: An enum of 'missing', 'placeholder', 'lowres', 'highres_scan'
        """
        super(CardsObject, self)._checkForKey('image_status')

        return self.scryfallJson['image_status']

    def finishes(self):
        """A list of computer-readable flags that indicate if this card
        can come in foil, nonfoil, etched, or glossy finishes.

        Returns:
            list: A list of all finishes.
        """
        super(CardsObject, self)._checkForKey('finishes')

        return self.scryfallJson['finishes']


import sys
sys.path.append('..')
from scrython.foundation import FoundationObject

class Code(FoundationObject):
    """
    sets/:code
    Get a set with a 3 letter code.

    Args:
        code (string): 
            The 3 letter code of the set.
        format (string, optional):
            Returns data in the specified method. Defaults to JSON.
        pretty (string, optional):
            Returns a prettier version of the json object. Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> set = scrython.sets.Code(code="por")
        >>> set.name()
    """
    def __init__(self, code):
        self._url = 'sets/{}?'.format(code)
        super(Code, self).__init__(self._url)

    def object(self):
        """Returns the type of object it is
        (card, error, etc)
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('object')

        return self.scryfallJson['object']

    def code(self):
        """The three letter set code of the set
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('object')

        return self.scryfallJson['code']

    def mtgo_code(self):
        """The mtgo equivalent of `code()`
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('mtgo_code')

        return self.scryfallJson['mtgo_code']

    def name(self):
        """The full name of the set
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('name')

        return self.scryfallJson['name']

    def set_type(self):
        """The type of the set (expansion, commander, etc)
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('set_type')

        return self.scryfallJson['set_type']

    def released_at(self):
        """The date the set was launched
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('released_at')

        return self.scryfallJson['released_at']

    def block_code(self):
        """The the letter code for the block the set was in
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('block_code')

        return self.scryfallJson['block_code']

    def block(self):
        """The full name of the block a set was in
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('block')

        return self.scryfallJson['block']

    def parent_set_code(self):
        """The set code for the parent set
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('parent_set_code')

        return self.scryfallJson['parent_set_code']

    def card_count(self):
        """The number of cards in the set
        
        Returns:
            integer
        """
        super(Code, self)._checkForKey('card_count')

        return self.scryfallJson['card_count']

    def digital(self):
        """True if this set is only featured on MTGO
        
        Returns:
            boolean
        """
        super(Code, self)._checkForKey('digital')

        return self.scryfallJson['digital']

    def foil_only(self):
        """True if this set only has foils
        
        Returns:
            boolean
        """
        super(Code, self)._checkForKey('foil_only')

        return self.scryfallJson['foil_only']

    def icon_svg_uri(self):
        """A URI to the SVG of the set symbol
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('icon_svg_uri')

        return self.scryfallJson['icon_svg_uri']

    def search_uri(self):
        """The scryfall API url for the search
        
        Returns:
            string
        """
        super(Code, self)._checkForKey('search_uri')

        return self.scryfallJson['search_uri']



from .cards_object import CardsObject
import urllib.parse

class Named(CardsObject):
    """
    cards/named
    Gets a card by the name.

    Args:
        fuzzy (string): Uses the fuzzy parameter for the card name.
        exact (string): Uses the exact parameter for the card name.
        set (string, optional):
            Defaults to empty string.
            Returns the set of the card if specified.
            Requires the 3 letter set code.
        format (string, optional):
            Defaults to 'json'.
            Returns data in the specified method.
        face (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify if you want the front or back face.
        version (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify if you want the small, normal,
            large, etc version of the image.
        pretty (string, optional):
            Defaults to empty string.
            Returns a prettier version of the json object.
            Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        Exception: If the 'fuzzy' or 'exact' parameter is not provided.
        Exception: If the object returned is an error.

    Examples:
        >>> card = scrython.cards.Named(exact="Black Lotus")
        >>> card.colors()
    """
    def __init__(self, **kwargs):

        self.dict = {
            'set':kwargs.get('set', '')
        }

        if 'exact' in kwargs:
            self.dict['exact'] = kwargs.get('exact')
        elif 'fuzzy' in kwargs:
            self.dict['fuzzy'] = kwargs.get('fuzzy')
        else:
            raise Exception('You must provide a `fuzzy` or `exact` parameter.')

        self.args = urllib.parse.urlencode(self.dict)
        self.url = 'cards/named?' + self.args
        super(Named, self).__init__(self.url)



from .catalogs_object import CatalogsObject


class KeywordAbilities(CatalogsObject):
    """
    catalog/keyword-abilities

    Catalog object for all known keyword abilities

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> catalog = scrython.catalog.KeywordAbilities()
        >>> catalog.data()
    """

    def __init__(self):
        self._url = 'catalog/keyword-abilities?'
        super(KeywordAbilities, self).__init__(self._url)



import aiohttp
import asyncio
import urllib

class ScryfallError(Exception):
    def __init__(self, error_obj, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.error_details = {}
        self.error_details.update(error_obj)


class FoundationObject(object):

    def __init__(self, _url, override=False, **kwargs):
        self.params = {
            'format': kwargs.get('format', 'json'), 'face': kwargs.get('face', ''),
            'version': kwargs.get('version', ''), 'pretty': kwargs.get('pretty', '')
        }

        self.encodedParams = urllib.parse.urlencode(self.params)
        self._url = 'https://api.scryfall.com/{0}&{1}'.format(_url, self.encodedParams)

        if override:
            self._url = _url

        async def getRequest(client, url, **kwargs):
            async with client.get(url, **kwargs) as response:
                return await response.json()

        async def main(loop):
            async with aiohttp.ClientSession(loop=loop) as client:
                self.scryfallJson = await getRequest(client, self._url)

        loop = asyncio.get_event_loop()
        loop.run_until_complete(main(loop))

        if self.scryfallJson['object'] == 'error':
            raise ScryfallError(self.scryfallJson, self.scryfallJson['details'])

    def _checkForKey(self, key, nested_key=None):
        """Checks for a key in the scryfallJson object.
        This function should be considered private, and
        should not be accessed in production.
        
        Args:
            key (string): The key to check
        
        Raises:
            KeyError: If key is not found.
        """
        if not key in self.scryfallJson:
            raise KeyError('This card has no key \'{}\''.format(key))

        if nested_key:
            if not nested_key in self.scryfallJson[key]:
                raise KeyError('This card has no key \'{}.{}\''.format(key, nested_key))


    def _checkForTupleKey(self, parent, num, key):
        """Checks for a key of an object in an array.
        This function should be considered private, and
        should not be accessed in production.
        
        Args:
            parent (string): The key for the array to be accessed
            num (int): The index of the array
            key (string): The key to check
        
        Raises:
            KeyError: If key is not found.
        """
        if not key in self.scryfallJson[parent][num]:
            raise KeyError('This tuple has no key \'{}\''.format(key))



from .rulings_object import RulingsObject

class Id(RulingsObject):
    """
    cards/:id/rulings

    Gets the ruling of a card by the Scryfall Id.

    Args:
        id (string): The id of the card you want rulings for.
        format (string, optional): Returns data in the specified method. Defaults to JSON.
        face (string, optional): 
            If you're using the `image` format, this will specify if you want the front or back face.
        version (string, optional):
            If you're using the `image` format, this will specify if you want the small, normal, large, etc version of the image.
        pretty (string, optional):
            Returns a prettier version of the json object. Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> rule = scrython.rulings.Id(id="5976c352-ac49-4e0d-a4c0-ec9b6b78db9c")
        >>> rule.data_length()
    """
    def __init__(self, **kwargs):
        if kwargs.get('id') is None:
            raise TypeError('No id provided to search by')

        self.url = 'cards/{}/rulings?'.format(str(kwargs.get('id')))
        super(Id, self).__init__(self.url)



import sys
sys.path.append('..')
from scrython.foundation import FoundationObject
import urllib.parse

class Autocomplete(FoundationObject):
    """
    cards/autocomplete
    Get a list of potential autocompletion phrases.

    Args:
        q (string):
            The query of the autocompletion.
        format (string, optional):
            Defaults to 'json'.
            Returns data in the specified method.
        face (string, optional):
            Defaults to empty string.
            If you're using the `image` format,
            this will specify if you want the front or back face.
        version (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify
            if you want the small, normal, large, etc version of the image.
        pretty (string, optional): 
            Defaults to empty string.
            Returns a prettier version of the json object. 
            Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        Exception: If the 'q' parameter is not provided.
        Exception: If the object returned is an error.

    Examples:
        >>> auto = scrython.cards.Autocomplete(q="Thal")
        >>> auto.total_items()
    """
    def __init__(self, **kwargs):
        if kwargs.get('q') is None:
            raise Exception('No query provided to search by')

        self.dict = { 'q': kwargs.get('q') }

        self.args = urllib.parse.urlencode(self.dict)
        self.url = 'cards/autocomplete?' + self.args
        super(Autocomplete, self).__init__(self.url)

    def object(self):
        """Returns the type of object it is.
        (card, error, etc)
        
        Returns:
            string: The type of object
        """
        super(Autocomplete, self)._checkForKey('object')

        return self.scryfallJson['object']

    def total_values(self):
        """How many items are returned in `data`
        
        Returns:
            int: The number of items in the `data` key
        """
        super(Autocomplete, self)._checkForKey('total_values')

        return self.scryfallJson['total_values']

    def data(self):
        """The list of potential autocompletes
        
        Returns:
            list: A list of possible corrections
        """
        super(Autocomplete, self)._checkForKey('data')

        return self.scryfallJson['data']


from .catalogs_object import CatalogsObject

class ArtifactTypes(CatalogsObject):
    """
    catalogs/artifact-types

    Catalog object for all known artifact types

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> catalog = scrython.catalog.ArtifactTypes()
        >>> catalog.data()
    """
    def __init__(self):
        self._url = 'catalog/artifact-types?'
        super(ArtifactTypes, self).__init__(self._url)



import sys
sys.path.append('..')
from scrython.foundation import FoundationObject

class Symbology(FoundationObject):
    """
    /symbology

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> symbol = scrython.symbology.Symbology()
    """
    def __init__(self):
        self.url = 'symbology?'
        super(Symbology, self).__init__(self.url)

    def object(self):
        """Returns the type of object it is
        (card, error, etc)

        Returns:
            string
        """
        super(Symbology, self)._checkForKey('object')

        return self.scryfallJson['object']

    def has_more(self):
        """True if there are more pages to the object
        
        Returns:
            boolean
        """
        super(Symbology, self)._checkForKey('has_more')

        return self.scryfallJson['has_more']

    def data(self, index=None, key=None):
        """The data returned from the query

        Acceptable keys:
            symbol (string): The plaintext symbol, usually written with curly braces
            loose_variant (string): The alternate version of the symbol, without curly braces
            transposable (boolean): True if it's possibly to write the symbol backwards
            represents_mana (boolean): True if this is a mana symbol
            cmc (float): The total converted mana cost of the symbol
            appears_in_mana_costs (boolean): True if the symbol appears on the mana cost of any card
            funny (boolean): True if the symbol is featured on any funny cards
            colors (array): An array of all colors in the given symbol
            english (string): An english sentence describing the mana cost
            gatherer_alternate (array): An array of Gatherer like costs

        Args:
            index (integer, optional): Defaults to None. Access a specific index.
            key (string, optional): Defaults to None. Returns the value of the given key. Requires the `index` argument.

        Returns:
            List: The full list of data.
            Dictionary: If given an index
            String: If given an index and key.
        """
        super(Symbology, self)._checkForKey('has_more')

        if index is not None:
            if key is not None:
                super(Symbology, self)._checkForTupleKey('data', index, key)
                return self.scryfallJson['data'][index][key]

            return self.scryfallJson['data'][index]

        return self.scryfallJson['data']

    def data_length(self):
        """The length of the data returned
        
        Returns:
            integer
        """
        super(Symbology, self)._checkForKey('data')

        return len(self.scryfallJson['data'])


import sys
sys.path.append('..')
from scrython.foundation import FoundationObject
from .cards_object import CardsObject
import urllib.parse

class Search(FoundationObject):
    """
    cards/search
    Uses a search query to gather relevant data.

    Args:
        q (string):
            The query to search. This will be updated in the future.
        order (string, optional):
            Defaults to 'none'
            The order you'd like the data returned.
        unique (string, optional):
            Defaults to 'none'
            A way to filter similar cards.
        dir (string, optional)
            Defaults to 'none'
            The direction you'd like to sort. (asc, desc, auto)
        include_extras (boolean, optional): 
            Defaults to 'false'
            Includes cards that are normally omitted from search results, like Un-sets.
        include_multilingual (boolean, optional):
            Defaults to 'false'
            Includes cards that are in the language specified. (en, ja, etc).
        page (integer, optional):
            Defaults to '1'
            The page number you'd like to search, if any.
        format (string, optional):
            Defaults to 'json'.
            Returns data in the specified method.
        face (string, optional):
            Defaults to empty string.
            If you're using the `image` format,
            this will specify if you want the front or back face.
        version (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify
            if you want the small, normal, large, etc version of the image.
        pretty (string, optional): 
            Defaults to empty string.
            Returns a prettier version of the json object. 
            Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        Exception: If the 'q' parameter is not provided.
        Exception: If the object returned is an error.

    Examples:
        >>> search = scrython.cards.Search(q="++e:A25", order="spoiled")
        >>> search.data()
    """
    def __init__(self, **kwargs):
        if kwargs.get('q') is None:
            raise Exception('No query is specified.')

        self.dict = {
            'q':kwargs.get('q'),
            'order':kwargs.get('order', 'none'),
            'unique':kwargs.get('unique', 'none'),
            'dir':kwargs.get('dir', 'none'),
            'include_variations':kwargs.get('include_variations','false'),
            'include_extras':kwargs.get('include_extras', 'false'),
            'include_multilingual':kwargs.get('include_multilingual', 'false'),
            'page':kwargs.get('page', '1')
            }
        self.args = urllib.parse.urlencode(self.dict)
        self.url = 'cards/search?' + self.args

        super(Search, self).__init__(self.url)

    def object(self):
        """Returns the type of object it is.
        (card, error, etc)
        
        Returns:
            string: The type of object
        """
        super(Search, self)._checkForKey('object')

        return self.scryfallJson['object']

    def total_cards(self):
        """How many cards are returned from the query
        
        Returns:
            integer: The number of cards returned
        """
        super(Search, self)._checkForKey('total_cards')

        return self.scryfallJson['total_cards']

    def data(self, index=None, key=None):
        """The data returned from the query

        You may reference any keys that could be accessed in a card object.
        There are far too many to list here, but you may find a list if applicable
        keys in the documentation.

        Args:
            index (integer, optional): Defaults to None. Access a specific index.
            key (string, optional): Defaults to None. Returns the value of the given key. Requires the `index` argument.
        
        Returns:
            List: The full list of data.
            Dictionary: If given an index.
            String: If given an index and key.
        """
        super(Search, self)._checkForKey('data')

        if index is not None:
            if key is not None:
                super(Search, self)._checkForTupleKey('data', index, key)
                return self.scryfallJson['data'][index][key]

            return self.scryfallJson['data'][index]

        return self.scryfallJson['data']

    def next_page(self):
        """The API URI to the next page of the query
        
        Returns:
            string: A URI to the next page of the query
        """
        super(Search, self)._checkForKey('next_page')

        return self.scryfallJson['next_page']

    def data_length(self):
        """
        
        Returns:
            integer: The length of data returned
        """
        super(Search, self)._checkForKey('data')

        return len(self.scryfallJson['data'])

    def has_more(self):
        """Determines if there are more pages of results.
        
        Returns:
            boolean: True if there is more than 1 page of results
        """
        super(Search, self)._checkForKey('has_more')

        return self.scryfallJson['has_more']



from .catalogs_object import CatalogsObject

class ArtistNames(CatalogsObject):
    """
    catalogs/card-names

    Catalog object for all known card names.

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> catalog = scrython.catalog.CardNames()
        >>> catalog.data()
    """
    def __init__(self):
        self._url = 'catalog/artist-names?'
        super(ArtistNames, self).__init__(self._url)



from .cards_object import CardsObject

class Id(CardsObject):
    """
    cards/id
    Get a card by the Scryfall id.

    Args:
        id (string):
            The Scryfall Id of the card.
        format (string, optional):
            Defaults to 'json'.
            Returns data in the specified method.
        face (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify if you want the front or back face.
        version (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify if you want the small, normal,
            large, etc version of the image.
        pretty (string, optional):
            Defaults to empty string.
            Returns a prettier version of the json object.
            Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        Exception: If the 'id' parameter is not provided.
        Exception: If the object returned is an error.

    Examples:
        >>> card = scrython.cards.Id(id="5386a61c-4928-4bd1-babe-5b089ab8b2d9")
        >>> card.name()
    """
    def __init__(self, **kwargs):
        if kwargs.get('id') is None:
            raise Exception('No id provided to search by')

        self.url = 'cards/{}?'.format(str(kwargs.get('id')))
        super(Id, self).__init__(self.url)



from setuptools import setup

setup(
	name='scrython',
	packages=['scrython', 'scrython.cards', 'scrython.rulings', 'scrython.catalog', 'scrython.sets', 'scrython.symbology', 'scrython.bulk_data'],
	version='1.11.0',
	description='A wrapper for using the Scryfall API.',
	long_description='https://github.com/NandaScott/Scrython/blob/master/README.md',
	url='https://github.com/NandaScott/Scrython',
	download_url='https://github.com/NandaScott/Scrython/archive/0.1.0.tar.gz',
	author='Nanda Scott',
	author_email='nanda1123@gmail.com',
	license='MIT',
	keywords=['Scryfall', 'magic', 'the gathering', 'scrython', 'wrapper'],
	install_requires=['aiohttp', 'asyncio']
	)



from .catalogs_object import CatalogsObject

class EnchantmentTypes(CatalogsObject):
    """
    catalogs/enchantment-types

    Catalog object for all known enchantment types.

    Args:
        N/A

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> catalog = scrython.catalog.EnchantmentTypes()
        >>> catalog.data()
    """
    def __init__(self):
        self._url = 'catalog/enchantment-types?'
        super(EnchantmentTypes, self).__init__(self._url)



from .cards_object import CardsObject


class Random(CardsObject):
    """
    cards/random
    Get a random card.

    Args:
        format (string, optional):
            Defaults to 'json'.
            Returns data in the specified method.
        face (string, optional):
            Defaults to empty string.
            If you're using the `image` format,
            this will specify if you want the front or back face.
        version (string, optional):
            Defaults to empty string.
            If you're using the `image` format, this will specify
            if you want the small, normal, large, etc version of the image.
        pretty (string, optional): 
            Defaults to empty string.
            Returns a prettier version of the json object. 
            Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        Exception: If the object returned is an error.

    Examples:
        >>> card = scrython.cards.Random()
        >>> card.purchase_uris()
    """
    def __init__(self):
        self.url = 'cards/random?'
        super(Random, self).__init__(self.url)



from .rulings_object import RulingsObject

class Code(RulingsObject):
    """
    cards/:code/:collector_number/rulings

    Gets the ruling of a card by the set and collector number.

    Args:
        set (string): The 3 letter set code of the card you want rulings for.
        collector_number (string): The collector number of the card.
        format (string, optional): Returns data in the specified method. Defaults to JSON.
        face (string, optional): 
            If you're using the `image` format, this will specify if you want the front or back face.
        version (string, optional):
            If you're using the `image` format, this will specify if you want the small, normal, large, etc version of the image.
        pretty (string, optional):
            Returns a prettier version of the json object. Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> rule = scrython.rulings.Code(code="ddg", collector_number="42")
        >>> rule.data_length()
    """
    def __init__(self, code, collector_number):
        self.url = 'cards/{}/{}/rulings?'.format(code.lower(), str(collector_number))
        super(Code, self).__init__(self.url)



import sys
sys.path.append('..')
from scrython.foundation import FoundationObject

class ParseMana(FoundationObject):
    """
    symbology/parse-mana

    Args:
        cost (string): The given mana cost you want. (`RUG`)
        format (string, optional):
            Returns data in the specified method. Defaults to JSON.
        pretty (string, optional):
            Returns a prettier version of the json object. Note that this may break functionality with Scrython.

    Returns:
        N/A

    Raises:
        N/A

    Examples:
        >>> mana = scrython.symbology.ParseMana(cost="xcug")
        >>> mana.colors()
    """
    def __init__(self, cost):
        self.cost = cost
        self.url = 'symbology/parse-mana?cost=' + self.cost
        super(ParseMana, self).__init__(self.url)

    def object(self):
        """Returns the type of object it is
        (card, error, etc)

        Returns:
            string
        """
        super(ParseMana, self)._checkForKey('object')

        return self.scryfallJson['object']

    def mana_cost(self):
        """The formatted mana cost
        
        Returns:
            string
        """
        super(ParseMana, self)._checkForKey('cost')

        return self.scryfallJson['cost']

    def cmc(self):
        """The converted mana cost of the card 
        
        Returns:
            float
        """
        super(ParseMana, self)._checkForKey('cmc')

        return self.scryfallJson['cmc']

    def colors(self):
        """A list of all colors in the mana cost
        
        Returns:
            list
        """
        super(ParseMana, self)._checkForKey('colors')

        return self.scryfallJson['colors']

    def colorless(self):
        """True if the mana cost is colorless
        
        Returns:
            boolean
        """
        super(ParseMana, self)._checkForKey('colorless')

        return self.scryfallJson['colorless']

    def monocolored(self):
        """True if the mana cost is mono colored
        
        Returns:
            boolean
        """
        super(ParseMana, self)._checkForKey('monocolored')

        return self.scryfallJson['monocolored']

    def multicolored(self):
        """True if the mana cost is a multicolored cost
        
        Returns:
            boolean
        """
        super(ParseMana, self)._checkForKey('multicolored')

        return self.scryfallJson['multicolored']
